#include "page_swirl.h"
#include "board/board.h"
#include "app/app_key.h"
// #include "app/app_anim.h"
#include "page/page_manager.h"

#include <Adafruit_GFX.h>
#include <Adafruit_IS31FL3731.h>
#include "gfxfont.h"
#include "Fonts/FreeMono9pt7b.h"
#include "Fonts/TomThumb.h"
#include "Fonts/Org_01.h"
#include "Fonts/Picopixel.h"
// If you're using the full breakout...
extern Adafruit_IS31FL3731 matrix;
extern void iic_setPins(uint8_t pinSDA, uint8_t pinSCL);
extern void iic_enable(void);
extern void iic_disable(void);

extern SoftwareTimer timeoutTimer;

static SoftwareTimer selfTimer;

// The lookup table to make the brightness changes be more visible
static uint8_t sweep[] = {1, 2, 3, 4, 6, 8, 10, 15, 20, 30, 40, 60, 60, 40, 30, 20, 15, 10, 8, 6, 4, 3, 2, 1};

static void swirl(void)
{
   static uint8_t incr = 0;
   iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
   Wire.setClock(400000);
   for (uint8_t x = 0; x < 16; x++)
      for (uint8_t y = 0; y < 9; y++)
         matrix.drawPixel(x, y, sweep[(x + y + incr) % 24]);

   incr++;
   if (incr == 24)
   {
      incr = 0;
   }
}
uint8_t direction = 0;
static void text()
{
   static int8_t i = 5;
   int16_t x1, y1;
   uint16_t w, h;
   char buffer[20];
   sprintf(buffer, "%s", "LCSC:Modular-X");
   matrix.getTextBounds(buffer, 0, 0, &x1, &y1, &w, &h);

   iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
   Wire.setClock(400000);
   matrix.setTextWrap(false);
   matrix.setFont(&Org_01);

   matrix.setCursor(i, 5);
   matrix.setTextColor(16);
   matrix.clear();
   matrix.print(buffer);
   if (direction == 0)
   {
      i--;
      if (i < 16 - w - 5)
      {
         direction = 1;
      }
   }
   else
   {
      i++;
      if (i > 5)
      {
         direction = 0;
      }
   }
}

void image(uint8_t x, uint8_t y)
{
   iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
   Wire.setClock(400000);
   for (uint8_t j = 0; j < 9; j++)
   {
      for (uint8_t i = 0; i < 16; i++)
      {
         // matrix.drawPixel(i,j,gImage_img_gray[x+i+16*(j+y)]);
      }
   }
}

static int8_t xpos = 0;
void rect(void)
{
   iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
   Wire.setClock(400000);

   matrix.drawRect(xpos, xpos, 16 - 2 * xpos, 9 - 2 * xpos, direction ? 16 : 0);

   if (direction == 0)
   {
      xpos--;
      if (xpos < 0)
      {
         direction = 1;
      }
   }
   else
   {
      xpos++;
      if (xpos >= 8)
      {
         direction = 0;
      }
   }
}

void led(void)
{
   static uint8_t k = 0;
   iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
   Wire.setClock(400000);
   for (uint8_t j = 0; j < 9; j++)
   {
      for (uint8_t i = 0; i < 16; i++)
      {
         matrix.drawPixel(i, j, xpos * 4);
      }
   }
   if (direction == 0)
   {
      xpos--;
      if (xpos < 1)
      {
         direction = 1;
         xpos = 1;
      }
   }
   else
   {
      xpos++;
      if (xpos >= 16)
      {
         direction = 0;
         xpos = 16;
      }
   }
}

void emoj()
{
   iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
   Wire.setClock(400000);
   matrix.clear();
   matrix.drawBitmap(0, 0, &img_emoj[16 * xpos], 16, 8, 16);
   xpos++;
   if (xpos == 23)
   {
      xpos = 0;
   }
}
static int8_t item = 0;
static void task_time_update(TimerHandle_t xTimerID)
{
   switch (item)
   {
   case 0:
      swirl();
      break;
   case 1:
      text();
      break;
   case 2:
      emoj();
      break;
   case 3:
      rect();
      break;
   case 4:
      led();
      break;
   default:
      break;
   }
   //
}

void *page_swirl_enter(void *data)
{
   xpos = 0;
   direction = 0;
   //停止息屏超时定时器
   timeoutTimer.stop();
   // image();
   // fadeIn();
   selfTimer.begin(100, task_time_update);
   selfTimer.start();
   app_key_clean();
   manager_set_busy_state(false);
   return NULL;
}

//根据按键情况，对其进行不同操作处理
void page_swirl_loop(void *data)
{
   KEY_TYPE key = app_key_get();
   switch (key)
   {
   case KEY0_SHORT:
      // manager_switch(LEVEL_PREV);
      item--;
      if(item<0)
      {
         item = 4;
      }

      break;
   case KEY1_SHORT:
   item++;
   if(item==5)
   item=0;
      // manager_switch(LEVEL_NEXT);
      break;
   case KEY0_LONG:
    timeoutTimer.reset();
      timeoutTimer.start();
      manager_switch(LEVEL_PARENT);
      break;
   case KEY1_LONG:
      // manager_switch(LEVEL_CHILD);
      break;

   default:
      break;
   }
}

void page_swirl_exit(void *data)
{

   selfTimer.stop();
   selfTimer.~SoftwareTimer();
   iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
   Wire.setClock(400000);
   matrix.clear();
   // lv_task_del(task);
   //退出动画，进行切换
}

const unsigned char img_emoj[] = {

    0x44, 0x22, 0x28, 0x14, 0x10, 0x08, 0x28, 0x14, 0x44, 0x22, 0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\0.BMP",0*/
    0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x30, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x10, 0x08, 0x0F, 0xF0, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\1.BMP",0*/
    0x30, 0x0C, 0x48, 0x12, 0x48, 0x12, 0x48, 0x12, 0x30, 0x0C, 0x00, 0x00, 0x08, 0x00, 0x0F, 0xF0, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\2.BMP",0*/
    0x04, 0x20, 0x04, 0x20, 0x08, 0x10, 0x10, 0x08, 0x60, 0x06, 0x00, 0x00, 0x0F, 0xF0, 0x10, 0x08, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\3.BMP",0*/
    0x00, 0x00, 0x10, 0x08, 0x28, 0x14, 0x28, 0x14, 0x28, 0x14, 0x00, 0x00, 0x0F, 0xF0, 0x10, 0x08, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\4.BMP",0*/
    0x00, 0x00, 0x38, 0x14, 0x28, 0x08, 0x38, 0x14, 0x00, 0x00, 0x00, 0x00, 0x10, 0x08, 0x0F, 0xF0, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\5.BMP",0*/
    0x00, 0x00, 0x10, 0x08, 0x38, 0x1C, 0x10, 0x08, 0x00, 0x00, 0x00, 0x00, 0x10, 0x08, 0x0F, 0xF0, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\6.BMP",0*/
    0x03, 0xC0, 0x07, 0xE0, 0x06, 0x60, 0x06, 0x60, 0x00, 0xC0, 0x01, 0x80, 0x00, 0x00, 0x01, 0x80, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\7.BMP",0*/
    0x30, 0x0C, 0x48, 0x12, 0x68, 0x1A, 0x68, 0x1A, 0x30, 0x0C, 0x00, 0x00, 0x10, 0x08, 0x0F, 0xF0, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\8.BMP",0*/
    0x00, 0x00, 0x0C, 0x30, 0x7C, 0x3E, 0x0C, 0x30, 0x00, 0x00, 0x00, 0x00, 0x0F, 0xF0, 0x10, 0x08, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\9.BMP",0*/
    0x00, 0x00, 0x60, 0x06, 0x7C, 0x3E, 0x60, 0x06, 0x00, 0x00, 0x00, 0x00, 0x0F, 0xF0, 0x10, 0x08, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\10.BMP",0*/
    0x00, 0x00, 0x00, 0x00, 0x7C, 0x3E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0F, 0xF0, 0x10, 0x08, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\11.BMP",0*/
    0x00, 0x00, 0x38, 0x1C, 0x38, 0x1C, 0x38, 0x1C, 0x00, 0x00, 0x00, 0x00, 0x04, 0x20, 0x03, 0xC0, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\12.BMP",0*/
    0x00, 0x00, 0x00, 0x00, 0x44, 0x22, 0x7C, 0x3E, 0x38, 0x1C, 0x00, 0x00, 0x07, 0xE0, 0x00, 0x00, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\13.BMP",0*/
    0x38, 0x1C, 0x7C, 0x3E, 0x44, 0x22, 0x44, 0x22, 0x00, 0x00, 0x08, 0x10, 0x04, 0x20, 0x03, 0xC0, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\14.BMP",0*/
    0x00, 0x00, 0x70, 0x0E, 0x88, 0x11, 0xA8, 0x15, 0x88, 0x11, 0x70, 0x0E, 0x07, 0xE0, 0x08, 0x10, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\15.BMP",0*/
    0x00, 0x00, 0x08, 0x10, 0x14, 0x28, 0x22, 0x44, 0x00, 0x00, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\16.BMP",0*/
    0x18, 0x18, 0x24, 0x24, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A, 0x24, 0x24, 0x18, 0x18, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\17.BMP",0*/
    0x30, 0x0C, 0x78, 0x1E, 0x78, 0x1E, 0x48, 0x12, 0x30, 0x0C, 0x03, 0xC0, 0x02, 0x40, 0x04, 0x20, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\18.BMP",0*/
    0x30, 0x0C, 0x08, 0x10, 0x08, 0x10, 0x10, 0x08, 0x20, 0x04, 0x03, 0xC0, 0x01, 0x80, 0x00, 0x00, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\19.BMP",0*/
    0x00, 0x00, 0x10, 0x08, 0x28, 0x14, 0x28, 0x14, 0x10, 0x08, 0x02, 0x40, 0x01, 0x80, 0x00, 0x00, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\20.BMP",0*/
    0x00, 0x00, 0x42, 0x42, 0x24, 0x24, 0x18, 0x18, 0x18, 0x18, 0x24, 0x24, 0x42, 0x42, 0x00, 0x00, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\21.BMP",0*/
    0x00, 0x00, 0x30, 0x0C, 0x28, 0x14, 0x24, 0x24, 0x13, 0xC8, 0x08, 0x10, 0x04, 0x20, 0x03, 0xC0, /*"D:\GitHub_Proj\watch_sister\software\now_nrf52832\img\22.BMP",0*/

};