#include "page_clock.h"
#include "app/app_batt.h"
// #include "app/app_anim.h"
#include "app/app_key.h"
#include "board/board.h"

#include "page/page_include.h"

extern PCF8563_Class rtc;
extern RTC_Date date;

static SoftwareTimer dateTimer;
// The lookup table to make the brightness changes be more visible
uint8_t sweep[] = {1, 2, 3, 4, 6, 8, 10, 15, 20, 30, 40, 60, 60, 40, 30, 20, 15, 10, 8, 6, 4, 3, 2, 1};
static void task_time_update(TimerHandle_t xTimerID);

static void drawBattery(uint8_t bright)
{
    static uint8_t i = 0;
    uint8_t perc = app_batt_percent();
    uint8_t w = perc * 16 / 100;

    iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
    Wire.setClock(400000);
    matrix.drawFastHLine(0, 8, w, bright);
    matrix.drawFastHLine(w, 8, 16 - w, 0);
    if (app_batt_charged())
    {
        matrix.drawPixel(15, 8, (i ? (i = 0) : (i = 1)) * bright);
    }
}
static uint8_t old_hour;
static uint8_t old_minute;
static void drawClock(uint8_t bright)
{
    uint16_t x, y;
    x = 0;
    y = 2;

    //Serial.println(date.hour);
    // date.hour-=1;

    iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
    Wire.setClock(400000);
    matrix.setRotation(0);
    matrix.setTextWrap(false);

    matrix.setTextSize(1);
    matrix.setFont(&Org_01);
    if (old_hour != date.hour)
    {

        matrix.setTextColor(0); // quarter brightness

        if (old_hour % 12 == 0)
        {
            matrix.setCursor(0 + x, 4 + y);
            matrix.printf("%d", old_hour);
        }
        else
        {
            if (old_hour % 12 / 10 == 1)
            {
                matrix.setCursor(0 + x, 4 + y);
                matrix.printf("1");
            }
            if (old_hour % 12 % 10 == 1) //字体‘1’
            {
                matrix.setCursor(4 + x, 4 + y);
            }
            else
            {
                matrix.setCursor(2 + x, 4 + y);
            }

            matrix.printf("%d", old_hour % 12 % 10);
        }

        matrix.setTextColor(bright); // quarter brightness
        if (date.hour % 12 == 0)
        {
            matrix.setCursor(0 + x, 4 + y);
            matrix.printf("%d", date.hour);
        }
        else
        {
            if (date.hour % 12 / 10 == 1)
            {
                matrix.setCursor(0 + x, 4 + y);
                matrix.printf("1");
            }
            if (date.hour % 12 % 10 == 1) //字体‘1’
            {
                matrix.setCursor(4 + x, 4 + y);
            }
            else
            {
                matrix.setCursor(2 + x, 4 + y);
            }
            matrix.printf("%d", date.hour % 12 % 10);
        }
    }

    matrix.setFont(&TomThumb);
    if (old_minute != date.minute)
    {
        matrix.setCursor(9 + x, 5 + y);
        matrix.setTextColor(0); // quarter brightness
        matrix.printf("%02d", old_minute);

        matrix.setCursor(9 + x, 5 + y);
        matrix.setTextColor(bright); // quarter brightness
        matrix.printf("%02d", date.minute);
    }

    old_hour = date.hour;
    old_minute = date.minute;
}

static void drawColockFadeIn(void)
{

    uint16_t x, y;
    uint8_t bright=64;
    x = 0;
    y = 2;

    iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
    Wire.setClock(400000);
    matrix.setRotation(0);
    matrix.setTextWrap(false);

    for (uint8_t i = 0; i < bright; i++)
    {
        matrix.setTextSize(1);
        matrix.setFont(&Org_01);
        matrix.setTextColor(i); // quarter brightness

        if (old_hour % 12 == 0)
        {
            matrix.setCursor(0 + x, 4 + y);
            matrix.printf("%d", old_hour);
        }
        else
        {
            if (old_hour % 12 / 10 == 1)
            {
                matrix.setCursor(0 + x, 4 + y);
                matrix.printf("1");
            }
            if (old_hour % 12 % 10 == 1) //字体‘1’
            {
                matrix.setCursor(4 + x, 4 + y);
            }
            else
            {
                matrix.setCursor(2 + x, 4 + y);
            }

            matrix.printf("%d", old_hour % 12 % 10);
        }

        matrix.setFont(&TomThumb);
        matrix.setCursor(9 + x, 5 + y);
        matrix.setTextColor(i); // quarter brightness
        matrix.printf("%02d", old_minute);

        delay(10);
    }
}

static void drawColockFadeOut(void)
{

    uint16_t x, y;
    uint8_t bright=0;
    x = 0;
    y = 2;

    iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
    Wire.setClock(400000);
    matrix.setRotation(0);
    matrix.setTextWrap(false);

    for (uint8_t i = 16; i > bright; i--)
    {
        matrix.setTextSize(1);
        matrix.setFont(&Org_01);
        matrix.setTextColor(i); // quarter brightness

        if (old_hour % 12 == 0)
        {
            matrix.setCursor(0 + x, 4 + y);
            matrix.printf("%d", old_hour);
        }
        else
        {
            if (old_hour % 12 / 10 == 1)
            {
                matrix.setCursor(0 + x, 4 + y);
                matrix.printf("1");
            }
            if (old_hour % 12 % 10 == 1) //字体‘1’
            {
                matrix.setCursor(4 + x, 4 + y);
            }
            else
            {
                matrix.setCursor(2 + x, 4 + y);
            }

            matrix.printf("%d", old_hour % 12 % 10);
        }

        matrix.setFont(&TomThumb);
        matrix.setCursor(9 + x, 5 + y);
        matrix.setTextColor(i); // quarter brightness
        matrix.printf("%02d", old_minute);

        delay(10);
    }
}

// static void anim_in_ready_cb(lv_anim_t * a);
// static void anim_out_ready_cb(lv_anim_t * a);

// static void task_time_update(lv_task_t *t);

void *page_clock_enter(void *data)
{
    old_hour = 0;
    old_minute = 0;

    iic_setPins(PIN_SDA, PIN_SCL);
    Wire.setClock(100000);
    date = rtc.getDateTime();
    // for(uint8_t i=0;i<16;i++)
    // {
    //   drawClock(i);
    //   delay(100);
    // }
    drawClock(0);
    
    drawColockFadeIn();
    drawBattery(4);
    
    

    dateTimer.begin(500, task_time_update);
    dateTimer.start();

    app_key_clean();
    manager_set_busy_state(false);

    //进入动画，进行切换
    // obj_anim_in(scr,100,100,anim_in_ready_cb,NULL);
    return NULL;
}

// lv_obj_t * page_clock_event(RTC_Date *date)
// {
// //    if()
// }

void page_clock_exit(void *data)
{

    dateTimer.stop();
    dateTimer.~SoftwareTimer();

    
    drawBattery(0);
    drawColockFadeOut();
    iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
    Wire.setClock(400000);
    matrix.clear();

    // lv_task_del(task);
    // //退出动画，进行切换
    // obj_anim_out(scr,100,0,anim_out_ready_cb,NULL);
}

static void task_time_update(TimerHandle_t xTimerID)
{
    static uint8_t x = 0;
    static uint8_t hour, minute, second = 0;
    // uint16_t voltage = app_batt_volatge();
    // Serial.println(voltage);
    second++;
    if (second == 120)
    {
        second = 0;
    }
    if (second == 0)
    {
        iic_setPins(PIN_SDA, PIN_SCL);
        Wire.setClock(100000);
        date = rtc.getDateTime();

        iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
        Wire.setClock(400000);
        drawClock(16);
        drawBattery(10);

        // RTC_Date date = rtc.getDateTime();
        // lv_label_set_text_fmt(label_time, "%02d%02d", date.hour, date.minute);
        // lv_label_set_text_fmt(label_batt, "%d%%", app_batt_percent());
        // lv_label_set_text_fmt(label_vol, "%dmV", app_batt_volatge());
    }

    matrix.drawFastHLine(0, 7, 16, 0);
    matrix.drawPixel(x, 7, sweep[second % 16 + 4]);

    x++;
    if (x == 16)
    {
        x = 0;
    }

    // if (led_second != NULL)
    //   lv_led_toggle(led_second);
}

// static void anim_in_ready_cb(lv_anim_t *a)
// {
//   app_key_clean();
//   manager_set_busy_state(false);
// }

// static void anim_out_ready_cb(lv_anim_t *a)
// {
//   lv_obj_del(scr);
// }

//根据按键情况，对其进行不同操作处理
void page_clock_loop(void *data)
{
    // Serial.println("enter loop");
    KEY_TYPE key = app_key_get();
    switch (key)
    {
    case KEY0_SHORT:
        // Serial.println("prev");
        manager_switch(LEVEL_PREV);

        break;
    case KEY1_SHORT:
        // Serial.println("next");
        manager_switch(LEVEL_NEXT);

        break;
    case KEY0_LONG:
        //  manager_switch(LEVEL_PARENT);
        break;
    case KEY1_LONG:
        if (manager.cur->child != NULL)
        {
            manager_switch(LEVEL_CHILD);
        }

        break;

    default:
        break;
    }
}