#include "page_date.h"
#include "app/app_batt.h"
// #include "app/app_anim.h"
#include "app/app_key.h"
#include "board/board.h"

#include "page/page_include.h"

extern PCF8563_Class rtc;
extern RTC_Date date;

static SoftwareTimer dateTimer;
static void task_time_update(TimerHandle_t xTimerID);
static uint8_t direction = 0;
static int8_t xpos = 5;
static char *week[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};

static void drawDate(uint8_t bright)
{

    int16_t x1, y1;
    uint16_t w, h;
    char buffer[40];
    sprintf(buffer, "%04d-%02d-%02d %s", date.year, date.month, date.day,week[date.dayOfTheWeek()]);
    matrix.getTextBounds(buffer, 0, 0, &x1, &y1, &w, &h);

    iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
    Wire.setClock(400000);
    matrix.setRotation(0);
    matrix.setTextWrap(false);

    matrix.setTextSize(1);
    matrix.setFont(&Org_01);
    matrix.clear();
    matrix.setCursor(xpos, 5);
    matrix.setTextColor(bright);
    matrix.print(buffer);
    if (direction == 0)
    {
        xpos--;
        if (xpos < 16 - w - 5)
        {
            direction = 1;
        }
    }
    else
    {
        xpos++;
        if (xpos > 5)
        {
            direction = 0;
        }
    }
}

static void drawDateFadeIn(void)
{

 
    uint8_t bright = 16;
 

    iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
    Wire.setClock(400000);
    matrix.setRotation(0);
    matrix.setTextWrap(false);

    for (uint8_t i = 0; i < bright; i++)
    {

        drawDate(i);
        xpos = 0;
        delay(10);
    }
}

static void drawDateFadeOut(void)
{

    uint16_t x, y;
    uint8_t bright = 0;
     

    iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
    Wire.setClock(400000);
    matrix.setRotation(0);
    matrix.setTextWrap(false);

    for (uint8_t i = 16; i > bright; i--)
    {
        drawDate(i);
        delay(10);
    }
}

// static void anim_in_ready_cb(lv_anim_t * a);
// static void anim_out_ready_cb(lv_anim_t * a);

// static void task_time_update(lv_task_t *t);

void *page_date_enter(void *data)
{

    iic_setPins(PIN_SDA, PIN_SCL);
    Wire.setClock(100000);
    date = rtc.getDateTime();

    drawDateFadeIn();

    dateTimer.begin(70, task_time_update);
    dateTimer.start();

    app_key_clean();
    manager_set_busy_state(false);

    //进入动画，进行切换
    // obj_anim_in(scr,100,100,anim_in_ready_cb,NULL);
    return NULL;
}

// lv_obj_t * page_date_event(RTC_Date *date)
// {
// //    if()
// }

void page_date_exit(void *data)
{

    dateTimer.stop();
    dateTimer.~SoftwareTimer();

    drawDateFadeOut();
    iic_setPins(PIN_SDA_IS31, PIN_SCL_IS31);
    Wire.setClock(400000);
    matrix.clear();

    // lv_task_del(task);
    // //退出动画，进行切换
    // obj_anim_out(scr,100,0,anim_out_ready_cb,NULL);
}

static void task_time_update(TimerHandle_t xTimerID)
{
    drawDate(16);
}

//根据按键情况，对其进行不同操作处理
void page_date_loop(void *data)
{
    // Serial.println("enter loop");
    KEY_TYPE key = app_key_get();
    switch (key)
    {
    case KEY0_SHORT:
        // Serial.println("prev");
        manager_switch(LEVEL_PREV);

        break;
    case KEY1_SHORT:
        // Serial.println("next");
        manager_switch(LEVEL_NEXT);

        break;
    case KEY0_LONG:
        //  manager_switch(LEVEL_PARENT);
        break;
    case KEY1_LONG:
        if (manager.cur->child != NULL)
        {
            manager_switch(LEVEL_CHILD);
        }

        break;

    default:
        break;
    }
}